@@ -110,39 +110,23 @@ hasSelection (LPDISPATCH explorer)
     }
 
   int count = get_oom_int (selection, "Count");
-
-  if (count)
+  bool selected = false;
+  if (count==1)
     {
-      LPDISPATCH selectitem = NULL, mailitem = NULL;
-      bool selected = hasMailitemEventReadBeenCalled ();
-      if (count == 1)
-        {
-          // If we call this on a selection with more items
-          // Outlook sends an ItemLoad event for each mail
-          // in that selection.
-          selectitem = get_oom_object (selection, "Item(1)");
-          mailitem = get_object_by_id (selectitem, IID_MailItem);
-          if (!mailitem)
-            {
-              log_debug ("%s:%s: New selection is no mail.",
-                SRCNAME, __func__);
-              selected = false;
-            }
+      selected = hasMailitemEventReadBeenCalled ();
+      log_debug ("%s:%s: ReadEvent %s been called",
+            SRCNAME, __func__, selected ? "HAS": "has NOT");
+    }
+  else
+    {
+      log_debug ("%s:%s: %d Items selected return false to show insecure",
+            SRCNAME, __func__, count);
 
-          gpgol_release (mailitem);
-          gpgol_release (selectitem);
-        }
-      else
-        {
-          selected = false; // We can't show the security level for more than one => show insecure
-        }
-
-      gpgol_release (selection);
-      return selected;
+        selected = false; // We can't show the security level for none/more than one => show insecure
     }
 
   gpgol_release (selection);
-  return false;
+  return selected;
 }
 
 static DWORD WINAPI
