@@ -42,6 +42,8 @@
 #include "mimemaker.h"
 #include "oomhelp.h"
 #include "mail.h"
+#include "attachment.h"
+#include "cpphelp.h"
 
 #undef _
 #define _(a) utf8_gettext (a)
@@ -573,6 +575,7 @@ infer_content_type (const char * /*data*/, size_t /*datalen*/,
     const char *ct;
   } suffix_table[] =
     {
+      { 1, "msg",   "application/ms-tnef" },
       { 1, "3gp",   "video/3gpp" },
       { 1, "abw",   "application/x-abiword" },
       { 1, "ai",    "application/postscript" },
@@ -873,8 +876,7 @@ utf8_to_rfc2047b (const char *input)
 static int
 write_part (sink_t sink, const char *data, size_t datalen,
             const char *boundary, const char *filename, int is_mapibody,
-            const char *content_id = nullptr,
-            const char *added_headers = nullptr)
+            const char *content_id = nullptr)
 {
   int rc;
   const char *ct;
@@ -968,15 +970,6 @@ write_part (sink_t sink, const char *data, size_t datalen,
                                NULL)))
       return rc;
 
-  /* Add any injected additional headers */
-  if (added_headers)
-    {
-      if ((rc = write_multistring (sink, added_headers, nullptr)))
-        {
-          return rc;
-        }
-    }
-
   xfree(encoded_filename);
 
   /* Write delimiter.  */
@@ -1025,116 +1018,60 @@ count_usable_attachments (mapi_attach_item_t *table)
    content-id. */
 static int
 write_attachments (sink_t sink,
-                   LPMESSAGE message, mapi_attach_item_t *table,
+                   Mail *mail,
                    const char *boundary, int only_related)
 {
-  int idx, rc;
-  char *buffer;
-  size_t buflen;
+  TSTART;
+  int rc;
   bool warning_shown = false;
-
-  if (table)
-    for (idx=0; !table[idx].end_of_table; idx++)
-      {
-        if (table[idx].attach_type == ATTACHTYPE_UNKNOWN
-            && table[idx].method == ATTACH_BY_VALUE)
-          {
-            if (only_related == 1 && !table[idx].content_id)
-              {
-                continue;
-              }
-            else if (!only_related && table[idx].content_id)
-              {
-                continue;
-              }
-            buffer = mapi_get_attach (message, table+idx, &buflen);
-            if (!buffer)
-              log_debug ("Attachment at index %d not found\n", idx);
-            else
-              log_debug ("Attachment at index %d: length=%d\n", idx, (int)buflen);
-            if (!buffer)
-              return -1;
-            rc = write_part (sink, buffer, buflen, boundary,
-                             table[idx].filename, 0, table[idx].content_id);
-            if (rc)
-              {
-                log_error ("Write part returned err: %i", rc);
-              }
-            xfree (buffer);
-          }
-        else if (!only_related && !warning_shown
-                && table[idx].attach_type == ATTACHTYPE_UNKNOWN
-                && (table[idx].method == ATTACH_OLE
-                    || table[idx].method == ATTACH_EMBEDDED_MSG))
-          {
-            char *fmt;
-            log_debug ("%s:%s: detected OLE attachment. Showing warning.",
-                       SRCNAME, __func__);
-            gpgrt_asprintf (&fmt, _("The attachment '%s' is an Outlook item "
-                                    "which is currently unsupported in crypto mails."),
-                            table[idx].filename ?
-                            table[idx].filename : _("Unknown"));
-            std::string msg = fmt;
-            msg += "\n\n";
-            xfree (fmt);
-
-            gpgrt_asprintf (&fmt, _("Please encrypt '%s' with Kleopatra "
-                                    "and attach it as a file."),
-                            table[idx].filename ?
-                            table[idx].filename : _("Unknown"));
-            msg += fmt;
-            xfree (fmt);
-
-            msg += "\n\n";
-            msg += _("Send anyway?");
-            warning_shown = true;
-
-            if (gpgol_message_box (get_active_hwnd (),
-                                   msg.c_str (),
-                                   _("Sorry, that's not possible, yet"),
-                                   MB_APPLMODAL | MB_YESNO) == IDNO)
-              {
-                return -1;
-              }
-          }
-        else
-          {
-            log_debug ("%s:%s: Skipping unknown attachment at idx: %d type: %d"
-                       " with method: %d",
-                       SRCNAME, __func__, idx, table[idx].attach_type,
-                       table[idx].method);
-          }
-      }
-  return 0;
-}
-
-/* Returns 1 if all attachments are related. 2 if there is a
-   related and a mixed attachment. 0 if there are no other parts*/
-static int
-is_related (Mail *mail, mapi_attach_item_t *table)
-{
-  if (!mail || !mail->isHTMLAlternative () || !table)
+  for (auto attach: mail->plainAttachments ())
     {
-      return 0;
-    }
-
-  int related = 0;
-  int mixed = 0;
-  for (int idx = 0; !table[idx].end_of_table; idx++)
-    {
-      if (table[idx].content_id)
+      std::string cid = attach->get_content_id ();
+      if (only_related == 1 && !cid.size ())
         {
-          related = 1;
+          continue;
         }
-      else
+      else if (!only_related && cid.size ())
         {
-          mixed = 1;
+          continue;
+        }
+      std::string buf = attach->get_data ().toString ();
+      const auto name = attach->get_file_name ();
+      if (!warning_shown && !only_related && !buf.size ())
+        {
+          log_debug ("%s:%s: detected OLE attachment. Showing warning.",
+                     SRCNAME, __func__);
+          std::string msg = asprintf_s (_("The attachment '%s' is an Outlook item "
+                                          "which is currently unsupported in crypto mails."),
+                                          name.size () ?
+                                          name.c_str (): _("Unknown"));
+          msg += "\n\n";
+          msg += asprintf_s (_("Please encrypt '%s' with Kleopatra "
+                               "and attach it as a file."),
+                               name.size () ?
+                               name.c_str (): _("Unknown"));
+          msg += "\n\n";
+          msg += _("Send anyway?");
+          warning_shown = true;
+
+          if (gpgol_message_box (get_active_hwnd (),
+                                 msg.c_str (),
+                                 _("Sorry, that's not possible, yet"),
+                                 MB_APPLMODAL | MB_YESNO) == IDNO)
+            {
+              TRETURN -1;
+            }
+        }
+      rc = write_part (sink, buf.c_str (), buf.size (), boundary,
+                       name.c_str (), 0, cid.size () ? cid.c_str () : nullptr);
+      if (rc)
+        {
+          log_error ("Write part returned err: %i", rc);
         }
     }
-  return mixed + related;
+  TRETURN 0;
 }
 
-
 /* Delete all attachments from TABLE except for the one we just created */
 static int
 delete_all_attachments (LPMESSAGE message, mapi_attach_item_t *table)
@@ -1401,9 +1338,7 @@ add_body (Mail *mail, const char *boundary, sink_t sink,
       if (plain_body)
         {
           rc = write_part (sink, plain_body, strlen (plain_body),
-                           boundary, NULL, 1, nullptr,
-                           mail ? mail->protectedHeaders ().c_str () :
-                           nullptr);
+                           boundary, NULL, 1, nullptr);
         }
       /* Just the plain body or no body. We are done. */
       return rc;
@@ -1432,9 +1367,7 @@ add_body (Mail *mail, const char *boundary, sink_t sink,
 
   /* Now the plain body part */
   if ((rc = write_part (sink, plain_body, strlen (plain_body),
-                       alt_boundary, NULL, 1, nullptr,
-                       mail ? mail->protectedHeaders().c_str() :
-                       nullptr)))
+                       alt_boundary, NULL, 1, nullptr)))
     {
       TRACEPOINT;
       return rc;
@@ -1465,16 +1398,23 @@ add_body (Mail *mail, const char *boundary, sink_t sink,
 
 /* Add the body and attachments. Does multipart handling. */
 int
-add_body_and_attachments (sink_t sink, LPMESSAGE message,
-                          mapi_attach_item_t *att_table, Mail *mail,
-                          const char *body, int n_att_usable)
+add_body_and_attachments (sink_t sink, Mail *mail, const char *body)
 {
-  int related = is_related (mail, att_table);
+  if (!mail)
+    {
+      STRANGEPOINT;
+      TRETURN -1;
+    }
+  int related = mail->isMultipartRelated ();
   int rc = 0;
   char inner_boundary[BOUNDARYSIZE+1];
   char outer_boundary[BOUNDARYSIZE+1];
   *outer_boundary = 0;
   *inner_boundary = 0;
+  int n_att_usable = mail->plainAttachments ().size ();
+  const auto protected_headers = mail->protectedHeaders ();
+  const char *protected_headers_line = protected_headers.empty() ? "" :
+                                       "\tprotected-headers=\"v1\"\r\n";
 
   if (((body && n_att_usable) || n_att_usable > 1) && related == 1)
     {
@@ -1483,21 +1423,40 @@ add_body_and_attachments (sink_t sink, LPMESSAGE message,
       if ((rc=write_multistring (sink,
                                  "Content-Type: multipart/related;\r\n",
                                  "\tboundary=\"", outer_boundary, "\"\r\n",
+                                 protected_headers_line,
                                  "\r\n", /* <--- Outlook adds an extra line. */
                                  NULL)))
         return rc;
     }
-  else if ((body && n_att_usable) || n_att_usable > 1)
+  else if ((body && n_att_usable) || n_att_usable > 1 || !protected_headers.empty ())
     {
       generate_boundary (outer_boundary);
       if ((rc=write_multistring (sink,
                                  "Content-Type: multipart/mixed;\r\n",
                                  "\tboundary=\"", outer_boundary, "\"\r\n",
+                                 protected_headers_line,
                                  "\r\n", /* <--- Outlook adds an extra line. */
                                  NULL)))
         return rc;
     }
 
+  if (!protected_headers.empty ())
+    {
+      log_data ("Adding protected headers part:\n'%s'\n",
+                protected_headers.c_str ());
+      if ((rc=write_boundary (sink, outer_boundary, 0)))
+        {
+          return rc;
+        }
+      if ((rc=write_multistring (sink,
+                                 protected_headers.c_str (),
+                                 NULL)))
+        {
+          return rc;
+        }
+
+    }
+
   /* Only one part.  */
   if (*outer_boundary && related == 2)
     {
@@ -1530,7 +1489,7 @@ add_body_and_attachments (sink_t sink, LPMESSAGE message,
   if (!rc && n_att_usable && related)
     {
       /* Write the related attachments. */
-      rc = write_attachments (sink, message, att_table,
+      rc = write_attachments (sink, mail,
                               *inner_boundary? inner_boundary :
                               *outer_boundary? outer_boundary : NULL, 1);
       if (rc)
@@ -1556,7 +1515,7 @@ add_body_and_attachments (sink_t sink, LPMESSAGE message,
      */
   if (!rc && n_att_usable)
     {
-      rc = write_attachments (sink, message, att_table,
+      rc = write_attachments (sink, mail,
                               *outer_boundary? outer_boundary : NULL,
                               related ? 0 : 2);
     }
